@@ -81,6 +81,8 @@ void remove_##NAME##_hash_table(TYPE *container, struct file_buffer *entry, int
 #define HASH_SIZE 65536
 #define CALCULATE_HASH(n) ((n) & 0xffff)
 
+#define FALSE 0
+#define TRUE 1
 
 #define NEXT_BLOCK	1
 #define NEXT_FILE	2
@@ -89,9 +91,8 @@ void remove_##NAME##_hash_table(TYPE *container, struct file_buffer *entry, int
 #define QUEUE_CACHE	1
 #define GEN_CACHE	2
 #define WSYNC_CMD	3
-#define OSYNC_CMD	4
-#define RESET_CMD	5
-#define MAP_CMD		6
+#define RESET_CMD	4
+#define MAP_CMD		5
 
 /* struct describing a cache entry passed between threads */
 struct file_buffer {
@@ -139,6 +140,7 @@ struct file_buffer {
 	char next_state;
 	char buffer_type;
 	char hashed;
+	char alignment;
 	char data[0] __attribute__((aligned));
 };
 
@@ -311,4 +313,24 @@ static inline int cache_maxsize(struct file_buffer *entry)
 }
 
 
+static inline int is_sparse(struct file_buffer *entry)
+{
+	return entry->c_byte < 0;
+}
+
+
+static inline int set_sparse(struct file_buffer *entry, long long sparse)
+{
+	if(sparse > INT_MAX)
+		return FALSE;
+
+	entry->c_byte = -sparse;
+	return TRUE;
+}
+
+
+static inline int sparse_count(struct file_buffer *entry)
+{
+	return -entry->c_byte;
+}
 #endif
