@@ -1,8 +1,10 @@
+#ifndef DATE_H
+#define DATE_H
 /*
  * Create a squashfs filesystem.  This is a highly compressed read only
  * filesystem.
  *
- * Copyright (c) 2023
+ * Copyright (c) 2023, 2024
  * Phillip Lougher <phillip@squashfs.org.uk>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,3 +28,4 @@ extern long long read_bytes(int, void *, long long);
 
 #define TRUE 1
 #define FALSE 0
+#endif
