@@ -2,8 +2,8 @@
  * Read a squashfs filesystem.  This is a highly compressed read only
  * filesystem.
  *
- * Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
- * 2012, 2013, 2014, 2019, 2021, 2022
+ * Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012,
+ * 2013, 2014, 2019, 2021, 2022, 2023, 2024, 2025
  * Phillip Lougher <phillip@squashfs.org.uk>
  *
  * This program is free software; you can redistribute it and/or
@@ -43,6 +43,10 @@
 #include "mksquashfs.h"
 #include "xattr.h"
 #include "mksquashfs_error.h"
+#include "mksquashfs_help.h"
+#include "alloc.h"
+#include "caches-queues-lists.h"
+#include "virt_disk_pos.h"
 
 int read_block(int fd, long long start, long long *next, int expected,
 								void *block)
@@ -149,11 +153,8 @@ unsigned char *scan_inode_table(int fd, long long start, long long end,
 				"containing root inode\n", start);
 			*root_inode_block = bytes;
 		}
-		if(size - bytes < SQUASHFS_METADATA_SIZE) {
-			inode_table = realloc(inode_table, size += alloc_size);
-			if(inode_table == NULL)
-				MEM_ERROR();
-		}
+		if(size - bytes < SQUASHFS_METADATA_SIZE)
+			inode_table = REALLOC(inode_table, size += alloc_size);
 		TRACE("scan_inode_table: reading block 0x%llx\n", start);
 		byte = read_block(fd, start, &start, 0, inode_table + bytes);
 		if(byte == 0)
@@ -238,9 +239,7 @@ unsigned char *scan_inode_table(int fd, long long start, long long end,
 	get_guid(id_table[dir_inode->base.guid]);
 
 	/* allocate fragment to file mapping table */
-	file_mapping = calloc(sBlk->fragments, sizeof(struct append_file *));
-	if(file_mapping == NULL)
-		MEM_ERROR();
+	file_mapping = CALLOC(sBlk->fragments, sizeof(struct append_file *));
 
 	for(cur_ptr = inode_table; cur_ptr < inode_table + bytes; files ++) {
 		/*
@@ -305,10 +304,7 @@ unsigned char *scan_inode_table(int fd, long long start, long long end,
 				/* corrupted filesystem */
 				goto corrupted;
 
-			block_list = malloc(blocks * sizeof(unsigned int));
-			if(block_list == NULL)
-				MEM_ERROR();
-
+			block_list = MALLOC(blocks * sizeof(unsigned int));
 			SQUASHFS_SWAP_INTS(cur_ptr, block_list, blocks);
 
 			*uncompressed_file += inode.file_size;
@@ -325,6 +321,8 @@ unsigned char *scan_inode_table(int fd, long long start, long long end,
 				goto corrupted;
 			}
 
+			add_virt_disk(start, start);
+
 			add_file(start, inode.file_size, file_bytes,
 				block_list, blocks, inode.fragment,
 				inode.offset, frag_bytes);
@@ -348,6 +346,11 @@ unsigned char *scan_inode_table(int fd, long long start, long long end,
 
 			SQUASHFS_SWAP_LREG_INODE_HEADER(cur_ptr, &inode);
 
+			if(inode.file_size < 0) {
+				ERROR("File system corrupted - negative file size in inode\n");
+				goto corrupted2;
+			}
+
 			frag_bytes = inode.fragment == SQUASHFS_INVALID_FRAG ?
 				0 : inode.file_size % sBlk->block_size;
 			blocks = inode.fragment == SQUASHFS_INVALID_FRAG ?
@@ -366,10 +369,7 @@ unsigned char *scan_inode_table(int fd, long long start, long long end,
 				/* corrupted filesystem */
 				goto corrupted;
 
-			block_list = malloc(blocks * sizeof(unsigned int));
-			if(block_list == NULL)
-				MEM_ERROR();
-
+			block_list = MALLOC(blocks * sizeof(unsigned int));
 			SQUASHFS_SWAP_INTS(cur_ptr, block_list, blocks);
 
 			*uncompressed_file += inode.file_size;
@@ -386,6 +386,8 @@ unsigned char *scan_inode_table(int fd, long long start, long long end,
 				goto corrupted;
 			}
 
+			add_virt_disk(start, start);
+
 			add_file(start, inode.file_size, file_bytes,
 				block_list, blocks, inode.fragment,
 				inode.offset, frag_bytes);
@@ -633,7 +635,7 @@ struct compressor *read_super(int fd, struct squashfs_super_block *sBlk, char *s
 		ERROR("Filesystem on %s uses %s compression, this is "
 			"unsupported by this version\n", source, comp->name);
 		ERROR("Compressors available:\n");
-		display_compressors(stderr, "", "");
+		display_compressors();
 		goto failed_mount;
 	}
 
@@ -724,32 +726,30 @@ static unsigned char *squashfs_readdir(int fd, int root_entries,
 		__attribute__ ((aligned));
 	struct squashfs_dir_entry *dire = (struct squashfs_dir_entry *) buffer;
 	unsigned char *directory_table = NULL;
-	int byte, dir_count;
+	int dir_count;
 	long long start = sBlk->directory_table_start + directory_start_block;
 	long long last_start_block = start, size = dir_size, bytes = 0;
 
 	size += offset;
-	directory_table = malloc((size + SQUASHFS_METADATA_SIZE * 2 - 1) &
+	directory_table = MALLOC((size + SQUASHFS_METADATA_SIZE * 2 - 1) &
 		~(SQUASHFS_METADATA_SIZE - 1));
-	if(directory_table == NULL)
-		MEM_ERROR();
-
-	while(bytes < size) {
-		int expected = (size - bytes) >= SQUASHFS_METADATA_SIZE ?
-			SQUASHFS_METADATA_SIZE : 0;
 
+	for(; size - bytes >= SQUASHFS_METADATA_SIZE; bytes += SQUASHFS_METADATA_SIZE) {
 		TRACE("squashfs_readdir: reading block 0x%llx, bytes read so "
 			"far %lld\n", start, bytes);
 
-		last_start_block = start;
-		byte = read_block(fd, start, &start, expected, directory_table + bytes);
-		if(byte == 0) {
-			ERROR("Failed to read directory\n");
-			ERROR("Filesystem corrupted?\n");
-			free(directory_table);
-			return NULL;
-		}
-		bytes += byte;
+		if(read_block(fd, start, &start, SQUASHFS_METADATA_SIZE, directory_table + bytes) == 0)
+			goto read_error;
+	}
+
+	last_start_block = start;
+
+	if(size - bytes) {
+		TRACE("squashfs_readdir: reading block 0x%llx, bytes read so "
+			"far %lld\n", start, bytes);
+
+		if(read_block(fd, start, &start, 0, directory_table + bytes) == 0)
+			goto read_error;
 	}
 
 	if(!root_entries)
@@ -801,6 +801,12 @@ all_done:
 	*last_directory_block = (unsigned int) last_start_block -
 		sBlk->directory_table_start;
 	return directory_table;
+
+read_error:
+	ERROR("Failed to read directory\n");
+	ERROR("Filesystem corrupted?\n");
+	free(directory_table);
+	return NULL;
 }
 
 
@@ -809,13 +815,9 @@ static unsigned int *read_id_table(int fd, struct squashfs_super_block *sBlk)
 	int indexes = SQUASHFS_ID_BLOCKS(sBlk->no_ids);
 	long long index[indexes];
 	int bytes = SQUASHFS_ID_BYTES(sBlk->no_ids);
-	unsigned int *id_table;
+	unsigned int *id_table = MALLOC(bytes);
 	int res, i;
 
-	id_table = malloc(bytes);
-	if(id_table == NULL)
-		MEM_ERROR();
-
 	res = read_fs_bytes(fd, sBlk->id_table_start,
 		SQUASHFS_ID_BLOCK_BYTES(sBlk->no_ids), index);
 	if(res == 0) {
@@ -862,16 +864,12 @@ static struct squashfs_fragment_entry *read_fragment_table(int fd, struct squash
 	long long bytes = SQUASHFS_FRAGMENT_BYTES(sBlk->fragments);
 	int indexes = SQUASHFS_FRAGMENT_INDEXES(sBlk->fragments);
 	long long fragment_table_index[indexes];
-	struct squashfs_fragment_entry *fragment_table;
+	struct squashfs_fragment_entry *fragment_table = MALLOC(bytes);
 
 	TRACE("read_fragment_table: %u fragments, reading %d fragment indexes "
 		"from 0x%llx\n", sBlk->fragments, indexes,
 		sBlk->fragment_table_start);
 
-	fragment_table = malloc(bytes);
-	if(fragment_table == NULL)
-		MEM_ERROR();
-
 	res = read_fs_bytes(fd, sBlk->fragment_table_start,
 		SQUASHFS_FRAGMENT_INDEX_BYTES(sBlk->fragments),
 		fragment_table_index);
@@ -915,11 +913,7 @@ static squashfs_inode *read_inode_lookup_table(int fd, struct squashfs_super_blo
 	int indexes = SQUASHFS_LOOKUP_BLOCKS(sBlk->inodes);
 	long long index[indexes];
 	int res, i;
-	squashfs_inode *inode_lookup_table;
-
-	inode_lookup_table = malloc(lookup_bytes);
-	if(inode_lookup_table == NULL)
-		MEM_ERROR();
+	squashfs_inode *inode_lookup_table = MALLOC(lookup_bytes);
 
 	res = read_fs_bytes(fd, sBlk->lookup_table_start,
 		SQUASHFS_LOOKUP_BLOCK_BYTES(sBlk->inodes), index);
@@ -1038,10 +1032,7 @@ long long read_filesystem(char *root_name, int fd, struct squashfs_super_block *
 			goto error;
 
 		root_inode_start -= start;
-		*cinode_table = malloc(root_inode_start);
-		if(*cinode_table == NULL)
-			MEM_ERROR();
-
+		*cinode_table = MALLOC(root_inode_start);
 	       	res = read_fs_bytes(fd, start, root_inode_start, *cinode_table);
 		if(res == 0) {
 			ERROR("Failed to read inode table\n");
@@ -1049,10 +1040,7 @@ long long read_filesystem(char *root_name, int fd, struct squashfs_super_block *
 			goto error;
 		}
 
-		*cdirectory_table = malloc(*last_directory_block);
-		if(*cdirectory_table == NULL)
-			MEM_ERROR();
-
+		*cdirectory_table = MALLOC(*last_directory_block);
 		res = read_fs_bytes(fd, sBlk->directory_table_start,
 			*last_directory_block, *cdirectory_table);
 		if(res == 0) {
@@ -1061,18 +1049,12 @@ long long read_filesystem(char *root_name, int fd, struct squashfs_super_block *
 			goto error;
 		}
 
-		*data_cache = malloc(root_inode_offset + *root_inode_size);
-		if(*data_cache == NULL)
-			MEM_ERROR();
-
+		*data_cache = MALLOC(root_inode_offset + *root_inode_size);
 		memcpy(*data_cache, inode_table + root_inode_block,
 			root_inode_offset + *root_inode_size);
 
-		*directory_data_cache = malloc(*inode_dir_offset +
+		*directory_data_cache = MALLOC(*inode_dir_offset +
 			*inode_dir_file_size);
-		if(*directory_data_cache == NULL)
-			MEM_ERROR();
-
 		memcpy(*directory_data_cache, directory_table,
 			*inode_dir_offset + *inode_dir_file_size);
 
