@@ -29,6 +29,8 @@
 #define SQUASHFS_MINOR			0
 #define SQUASHFS_MAGIC			0x73717368
 #define SQUASHFS_MAGIC_SWAP		0x68737173
+#define SQUASHFS_MAGIC_STREAMED		0x53515348
+#define SQUASHFS_MAGIC_STREAMED_SWAPPED	0x48535153
 #define SQUASHFS_START			0
 
 /* size of metadata (inode and directory) blocks */
