@@ -2,7 +2,7 @@
  * Unsquash a squashfs filesystem.  This is a highly compressed read only
  * filesystem.
  *
- * Copyright (c) 2009, 2010, 2011, 2012, 2019, 2021, 2022
+ * Copyright (c) 2009, 2010, 2011, 2012, 2013, 2019, 2021, 2022, 2023, 2025
  * Phillip Lougher <phillip@squashfs.org.uk>
  *
  * This program is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@
 #include "unsquashfs.h"
 #include "squashfs_compat.h"
 #include "compressor.h"
+#include "alloc.h"
 
 static unsigned int *uid_table, *guid_table;
 static squashfs_operations ops;
@@ -37,16 +38,10 @@ static void read_block_list(unsigned int *block_list, long long start,
 
 	TRACE("read_block_list: blocks %d\n", blocks);
 
-	source = malloc(blocks * sizeof(unsigned short));
-	if(source == NULL)
-		MEM_ERROR();
+	source = MALLOC(blocks * sizeof(unsigned short));
 
 	if(swap) {
-		char *swap_buff;
-
-		swap_buff = malloc(blocks * sizeof(unsigned short));
-		if(swap_buff == NULL)
-			MEM_ERROR();
+		char *swap_buff = MALLOC(blocks * sizeof(unsigned short));
 
 		res = read_inode_data(swap_buff, &start, &offset, blocks * sizeof(unsigned short));
 		if(res == FALSE)
@@ -142,7 +137,6 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 	else
 		i.gid = (uid_t) guid_table[header.base.guid];
 
-	i.time = sBlk.s.mkfs_time;
 	i.inode_number = inode_number ++;
 
 	switch(i.type) {
@@ -165,7 +159,10 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 			i.data = inode->file_size;
 			i.offset = inode->offset;
 			i.start = inode->start_block;
-			i.time = inode->mtime;
+			if(time_opt)
+				i.time = timeval;
+			else
+				i.time = inode->mtime;
 			break;
 		}
 		case SQUASHFS_FILE_TYPE: {
@@ -185,7 +182,10 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 					"inode %lld:%d\n", start, offset);
 
 			i.data = inode->file_size;
-			i.time = inode->mtime;
+			if(time_opt)
+				i.time = timeval;
+			else
+				i.time = inode->mtime;
 			i.blocks = (i.data + sBlk.s.block_size - 1) >>
 				sBlk.s.block_log;
 			i.start = inode->start_block;
@@ -214,16 +214,17 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 				EXIT_UNSQUASH("read_inode: failed to read "
 					"inode %lld:%d\n", start, offset);
 
-			i.symlink = malloc(inodep->symlink_size + 1);
-			if(i.symlink == NULL)
-				MEM_ERROR();
-
+			i.symlink = MALLOC(inodep->symlink_size + 1);
 			res = read_inode_data(i.symlink, &start, &offset, inodep->symlink_size);
 			if(res == FALSE)
 				EXIT_UNSQUASH("read_inode: failed to read "
 					"inode symbolic link %lld:%d\n", start, offset);
 			i.symlink[inodep->symlink_size] = '\0';
 			i.data = inodep->symlink_size;
+			if(time_opt)
+				i.time = timeval;
+			else
+				i.time = sBlk.s.mkfs_time;
 			break;
 		}
  		case SQUASHFS_BLKDEV_TYPE:
@@ -244,11 +245,19 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 					"inode %lld:%d\n", start, offset);
 
 			i.data = inodep->rdev;
+			if(time_opt)
+				i.time = timeval;
+			else
+				i.time = sBlk.s.mkfs_time;
 			break;
 			}
 		case SQUASHFS_FIFO_TYPE:
 		case SQUASHFS_SOCKET_TYPE: {
 			i.data = 0;
+			if(time_opt)
+				i.time = timeval;
+			else
+				i.time = sBlk.s.mkfs_time;
 			break;
 			}
 		default:
@@ -278,10 +287,7 @@ static struct dir *squashfs_opendir(unsigned int block_start, unsigned int offse
 
 	*i = read_inode(block_start, offset);
 
-	dir = malloc(sizeof(struct dir));
-	if(dir == NULL)
-		MEM_ERROR();
-
+	dir = MALLOC(sizeof(struct dir));
 	dir->dir_count = 0;
 	dir->cur_entry = NULL;
 	dir->mode = (*i)->mode;
@@ -367,11 +373,8 @@ static struct dir *squashfs_opendir(unsigned int block_start, unsigned int offse
 				"%d:%d, type %d\n", dire->name,
 				dirh.start_block, dire->offset, dire->type);
 
-			ent = malloc(sizeof(struct dir_ent));
-			if(ent == NULL)
-				MEM_ERROR();
-
-			ent->name = strdup(dire->name);
+			ent = MALLOC(sizeof(struct dir_ent));
+			ent->name = STRDUP(dire->name);
 			ent->start_block = dirh.start_block;
 			ent->offset = dire->offset;
 			ent->type = dire->type;
@@ -387,7 +390,7 @@ static struct dir *squashfs_opendir(unsigned int block_start, unsigned int offse
 	}
 
 	/* check directory for duplicate names.  Need to sort directory first */
-	sort_directory(dir);
+	sort_directory(&(dir->dirs), dir->dir_count);
 	if(check_directory(dir) == FALSE) {
 		ERROR("File system corrupted: directory has duplicate names\n");
 		goto corrupted;
