@@ -2,7 +2,8 @@
  * Unsquash a squashfs filesystem.  This is a highly compressed read only
  * filesystem.
  *
- * Copyright (c) 2009, 2010, 2011, 2012, 2013, 2019, 2021, 2022
+ * Copyright (c) 2009, 2010, 2011, 2012, 2013, 2019, 2021, 2022, 2023, 2024,
+ * 2025
  * Phillip Lougher <phillip@squashfs.org.uk>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,6 +27,7 @@
 #include "squashfs_swap.h"
 #include "xattr.h"
 #include "compressor.h"
+#include "alloc.h"
 
 static struct squashfs_fragment_entry *fragment_table;
 static unsigned int *id_table;
@@ -77,10 +79,7 @@ static int read_fragment_table(long long *table_start)
 		sBlk.s.fragment_table_start);
 
 	fragment_table_index = alloc_index_table(indexes);
-	fragment_table = malloc(bytes);
-	if(fragment_table == NULL)
-		MEM_ERROR();
-
+	fragment_table = MALLOC(bytes);
 	res = read_fs_bytes(fd, sBlk.s.fragment_table_start, length,
 							fragment_table_index);
 	if(res == FALSE) {
@@ -158,13 +157,16 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 		EXIT_UNSQUASH("File system corrupted - inode number in inode too large (inode_number: %u)\n", header.base.inode_number);
 
 	if(header.base.inode_number == 0)
-		EXIT_UNSQUASH("File system corrupted - inode number zero is invalid\n", header.base.inode_number);
+		EXIT_UNSQUASH("File system corrupted - inode number zero is invalid\n");
 
 	i.uid = (uid_t) id_table[header.base.uid];
 	i.gid = (uid_t) id_table[header.base.guid];
 	i.mode = lookup_type[header.base.inode_type] | header.base.mode;
 	i.type = header.base.inode_type;
-	i.time = header.base.mtime;
+	if(time_opt)
+		i.time = timeval;
+	else
+		i.time = header.base.mtime;
 	i.inode_number = header.base.inode_number;
 
 	switch(header.base.inode_type) {
@@ -236,6 +238,9 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 
 			SQUASHFS_INSWAP_LREG_INODE_HEADER(inode);
 
+			if(inode->file_size < 0)
+				EXIT_UNSQUASH("File system corrupted - negative file size in inode\n");
+
 			i.data = inode->file_size;
 			i.frag_bytes = inode->fragment == SQUASHFS_INVALID_FRAG
 				?  0 : inode->file_size % sBlk.s.block_size;
@@ -266,10 +271,7 @@ static struct inode *read_inode(unsigned int start_block, unsigned int offset)
 			if(inode->symlink_size > SQUASHFS_SYMLINK_MAX)
 				EXIT_UNSQUASH("File system corrupted - symlink_size in inode too large (symlink_size: %u)\n", inode->symlink_size);
 
-			i.symlink = malloc(inode->symlink_size + 1);
-			if(i.symlink == NULL)
-				MEM_ERROR();
-
+			i.symlink = MALLOC(inode->symlink_size + 1);
 			res = read_inode_data(i.symlink, &start, &offset, inode->symlink_size);
 			if(res == FALSE)
 				EXIT_UNSQUASH("read_inode: failed to read "
@@ -360,15 +362,7 @@ static struct dir *squashfs_opendir(unsigned int block_start, unsigned int offse
 
 	*i = read_inode(block_start, offset);
 
-	if(inumber_lookup((*i)->inode_number)) {
-		ERROR("File System corrupted: directory loop detected\n");
-		return NULL;
-	}
-
-	dir = malloc(sizeof(struct dir));
-	if(dir == NULL)
-		MEM_ERROR();
-
+	dir = MALLOC(sizeof(struct dir));
 	dir->dir_count = 0;
 	dir->cur_entry = NULL;
 	dir->mode = (*i)->mode;
@@ -441,11 +435,8 @@ static struct dir *squashfs_opendir(unsigned int block_start, unsigned int offse
 				"%d:%d, type %d\n", dire->name,
 				dirh.start_block, dire->offset, dire->type);
 
-			ent = malloc(sizeof(struct dir_ent));
-			if(ent == NULL)
-				MEM_ERROR();
-
-			ent->name = strdup(dire->name);
+			ent = MALLOC(sizeof(struct dir_ent));
+			ent->name = STRDUP(dire->name);
 			ent->start_block = dirh.start_block;
 			ent->offset = dire->offset;
 			ent->type = dire->type;
@@ -501,13 +492,7 @@ static int read_id_table(long long *table_start)
 	TRACE("read_id_table: no_ids %d\n", sBlk.s.no_ids);
 
 	id_index_table = alloc_index_table(indexes);
-
-	id_table = malloc(bytes);
-	if(id_table == NULL) {
-		ERROR("read_id_table: failed to allocate id table\n");
-		return FALSE;
-	}
-
+	id_table = MALLOC(bytes);
 	res = read_fs_bytes(fd, sBlk.s.id_table_start, length, id_index_table);
 	if(res == FALSE) {
 		ERROR("read_id_table: failed to read id index table\n");
@@ -699,6 +684,21 @@ int read_super_4(squashfs_operations **s_ops)
 	if(res == FALSE)
 		return res;
 
+	if(sBlk_4.s_magic == SQUASHFS_MAGIC_STREAMED || sBlk_4.s_magic == SQUASHFS_MAGIC_STREAMED_SWAPPED) {
+		long long res = lseek(fd, - sizeof(struct squashfs_super_block), SEEK_END);
+
+		if(res == -1) {
+			ERROR("Lseek failed because %s\n", strerror(errno));
+			return FALSE;
+		}
+
+		res = read_bytes(fd, &sBlk_4, sizeof(struct squashfs_super_block));
+		if(res < sizeof(struct squashfs_super_block)) {
+			ERROR("Read on filesystem failed\n");
+			return FALSE;
+		}
+	}
+
 	swap = sBlk_4.s_magic != SQUASHFS_MAGIC;
 	SQUASHFS_INSWAP_SUPER_BLOCK(&sBlk_4);
 
@@ -796,7 +796,7 @@ static void squashfs_stat(char *source)
 		printf("Fragments are %scompressed\n",
 			SQUASHFS_UNCOMPRESSED_FRAGMENTS(sBlk.s.flags) ?
 			"un" : "");
-		printf("Always-use-fragments option is %sspecified\n",
+		printf("Tailends are %spacked into fragments\n",
 			SQUASHFS_ALWAYS_FRAGMENTS(sBlk.s.flags) ? "" : "not ");
 	}
 
