@@ -4,7 +4,7 @@
  * Unsquash a squashfs filesystem.  This is a highly compressed read only
  * filesystem.
  *
- * Copyright (c) 2009, 2010, 2013, 2014, 2019, 2021, 2022
+ * Copyright (c) 2009, 2010, 2012, 2013, 2014, 2019, 2021, 2022, 2023, 2024
  * Phillip Lougher <phillip@squashfs.org.uk>
  *
  * This program is free software; you can redistribute it and/or
@@ -49,6 +49,7 @@
 #include "endian_compat.h"
 #include "squashfs_fs.h"
 #include "unsquashfs_error.h"
+#include "unsquashfs_help.h"
 
 #define TABLE_HASH(start)	(start & 0xffff)
 
@@ -275,6 +276,7 @@ struct directory_stack {
 #define LOOKUP_INDEX(NUMBER)		((NUMBER) >> 12)
 #define LOOKUP_OFFSET(NUMBER)		((NUMBER) & 0xfff)
 #define LOOKUP_BYTES			32768
+#define LOOKUP_OFFSETS			4096
 
 /* Maximum transfer size for Linux read() call on both 32-bit and 64-bit systems.
  * See READ(2) */
@@ -294,6 +296,8 @@ extern struct queue *to_reader, *to_inflate, *to_writer;
 extern struct cache *fragment_cache, *data_cache;
 extern struct compressor *comp;
 extern int use_localtime;
+extern unsigned int timeval;
+extern int time_opt;
 
 /* unsquashfs.c */
 extern int read_inode_data(void *, long long *, unsigned int *, int);
@@ -304,6 +308,8 @@ extern void enable_progress_bar();
 extern void disable_progress_bar();
 extern void dump_queue(struct queue *);
 extern void dump_cache(struct cache *);
+extern int write_bytes(int, char *, int);
+extern long long read_bytes(int, void *, long long);
 
 /* unsquash-1.c */
 int read_super_1(squashfs_operations **, void *);
@@ -326,7 +332,7 @@ extern int inumber_lookup(unsigned int);
 extern void free_inumber_table();
 extern char *lookup(unsigned int);
 extern void insert_lookup(unsigned int, char *);
-extern void free_lookup_table();
+extern void free_lookup_table(int);
 
 /* unsquash-1234.c */
 extern int check_name(char *, int);
@@ -334,5 +340,8 @@ extern void squashfs_closedir(struct dir *);
 extern int check_directory(struct dir *);
 
 /* unsquash-12.c */
-extern void sort_directory(struct dir *);
+extern void sort_directory(struct dir_ent **, int);
+
+/* date.c */
+extern int exec_date(char *, unsigned int *);
 #endif
