@@ -4,7 +4,7 @@
  * Create a squashfs filesystem.  This is a highly compressed read only
  * filesystem.
  *
- * Copyright (c) 2010, 2012, 2013, 2014, 2019, 2021, 2022
+ * Copyright (c) 2010, 2012, 2013, 2014, 2019, 2021, 2022, 2023, 2024
  * Phillip Lougher <phillip@squashfs.org.uk>
  *
  * This program is free software; you can redistribute it and/or
@@ -80,16 +80,21 @@ struct xattr_add {
 	struct xattr_add	*next;
 };
 
-extern int generate_xattrs(int, struct xattr_list *);
-
 #ifdef XATTR_SUPPORT
+#ifdef XATTR_OS_SUPPORT
+extern int write_xattr(char *, unsigned int);
+#else
+static inline int write_xattr(char *pathname, unsigned int xattr)
+{
+	return 1;
+}
+#endif
 extern int get_xattrs(int, struct squashfs_super_block *);
 extern int read_xattrs(void *, int type);
 extern long long write_xattrs();
 extern void save_xattrs();
 extern void restore_xattrs();
 extern unsigned int xattr_bytes, total_xattr_bytes;
-extern int write_xattr(char *, unsigned int);
 extern unsigned int read_xattrs_from_disk(int, struct squashfs_super_block *, int, long long *);
 extern struct xattr_list *get_xattr(int, unsigned int *, int *);
 extern void free_xattr(struct xattr_list *, int);
@@ -99,7 +104,8 @@ extern void sort_xattr_add_list(void);
 extern char *base64_decode(char *source, int size, int *bytes);
 extern int add_xattrs(void);
 extern struct xattr_add *xattr_parse(char *, char *, char *);
-extern int read_pseudo_xattr(char *orig_def, char *filename, char *name, char *def);
+extern struct xattr_add *read_pseudo_xattr(char *def);
+extern struct pseudo *add_pseudo_xattr_definition(struct pseudo *pseudo, struct xattr_add *xattr, char *target, char *alltarget);
 extern void print_xattr(char *, unsigned int, int);
 extern int has_xattrs(unsigned int);
 #else
@@ -201,12 +207,17 @@ static inline struct xattr_add *xattr_parse(char *a, char *b, char *c)
 }
 
 
-static inline int read_pseudo_xattr(char *orig_def, char *filename, char *name, char *def)
+static inline struct xattr_add *read_pseudo_xattr(char *def)
 {
-	free(filename);
 	fprintf(stderr, "Xattrs are unsupported in this build\n");
 
-	return 0;
+	return NULL;
+}
+
+
+static inline struct pseudo *add_pseudo_xattr_definition(struct pseudo *pseudo, struct xattr_add *xattr, char *target, char *alltarget)
+{
+	return NULL;
 }
 
 
@@ -223,19 +234,21 @@ static inline int has_xattrs(unsigned int xattr)
 
 #ifdef XATTR_SUPPORT
 #define xattrs_supported() TRUE
+#ifdef XATTR_OS_SUPPORT
 #ifdef XATTR_DEFAULT
-#define NOXOPT_STR
-#define XOPT_STR " (default)"
 #define XATTR_DEF 0
 #else
-#define NOXOPT_STR " (default)"
-#define XOPT_STR
 #define XATTR_DEF 1
 #endif
 #else
+#ifdef XATTR_DEFAULT
+#define XATTR_DEF 0
+#else
+#define XATTR_DEF 1
+#endif
+#endif
+#else
 #define xattrs_supported() FALSE
-#define NOXOPT_STR " (default)"
-#define XOPT_STR " (unsupported)"
 #define XATTR_DEF 1
 #endif
 #endif
